<?php
/**
 * @package         LartsDownloads Pro
 * @version         24.3.0
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2024 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;

class DatabaseHelper
{
	public static function executeQuery($query)
	{
		$db = Factory::getDbo();
		$db->setQuery($query);
		if ($db->execute() != true) {
			return false;
		}
		return true;
	}

	// Get the fields from your database tables
	public static function getColumns($table)
	{
		$db = Factory::getDbo();

		$query = 'SHOW COLUMNS FROM ' . $db->quoteName($table);
		$db->setQuery($query);
		$results = $db->loadAssocList();

		$columns = [];

		foreach ($results as $value)
		{
			$columns[] = $value['Field'];
		}

		return $columns;
	}

	public static function dropColumn($table, $column)
	{
		// Check the categories table:
		$columns = $this->getColumns($table);

		if (!in_array($column, $columns))
		{
			return;
		}

		$db = Factory::getDbo();

		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' DROP ' . $db->quoteName($column) . '';

		return self::executeQuery($query);
	}

	/*
	 * ALTER TABLE `$table`
	 * ADD `$column` $datatype AFTER `$after`,
	 */
	public static function addColumn($table, $column, $datatype, $after = '')
	{
		// Check the categories table:
		$columns = $this->getColumns($table);

		if (in_array($column, $columns)) {
			return;
		}

		$db = Factory::getDbo();

		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' ADD ' . $db->quoteName($column) . ' ' . $datatype;

		if (!empty($after)) {
			$query .= ' AFTER ' . $db->quoteName($after);
		}

		return self::executeQuery($query);
	}

	/*
	 * ALTER TABLE `$table`
	 * CHANGE COLUMN `$column` `$new_name` $datatype AFTER `$after`,
	 */
	public static function renameColumn($table, $column, $new_name, $datatype, $after = '')
	{
		// Check the categories table:
		$columns = $this->getColumns($table);

		if (!in_array($column, $columns) || in_array($new_name, $columns))
		{
			return;
		}

		$db = Factory::getDbo();

		$query = 'ALTER TABLE ' . $db->quoteName($table)
			. ' CHANGE COLUMN  ' . $db->quoteName($column) . ' ' . $db->quoteName($new_name) . ' ' . $datatype;

		if (!empty($after) && in_array($after, $columns))
		{
			$query .= ' AFTER ' . $db->quoteName($after);
		}

		return self::executeQuery($query);
	}

	public static function addIndex($table, $name, $columns = [])
	{
		$db = Factory::getDbo();

		if (empty($columns))
		{
			$columns = [$name];
		}

		array_walk($columns, function (&$column, $key, $db) {
			$column = $db->quoteName($column);
		}, $db);

		$query = 'ALTER TABLE ' . $db->quoteName($table) . ' DROP INDEX ' . $db->quoteName($name) . ';'
			. 'ALTER TABLE ' . $db->quoteName($table) . ' ADD INDEX ' . $db->quoteName($name) . ' (' . implode(',', $columns) . ');';

		return self::executeQuery($query);
	}
	
	public static function DropIndex($table, $index)
	{	
		$db = Factory::getDbo();

		// Check if index exists first
		$query = 'SHOW INDEX FROM ' . $db->quoteName('#__' . $table) . ' WHERE KEY_NAME = ' . $db->quote($index);
        $db->setQuery($query);
        $db->execute();

        if (!$db->loadResult()) {
        	return;
        }

        // Remove index
        $query = 'ALTER TABLE ' . $db->quoteName('#__' . $table) . ' DROP INDEX ' . $db->quoteName($index);
        $db->setQuery($query);
        $db->execute(); 
	}

    public function DropUnwantedTables($tables) {

        if (!$tables) {
            return;
        }

        foreach ($tables as $table) {
            $query = "DROP TABLE IF EXISTS #__".$this->db->escape($table);
            $this->db->setQuery($query);
            $this->db->execute();
        }
    }

	public static function DropUnwantedColumns($table, $columns)
	{
		if (!$columns || !$table) {
				return;
		}

		$db = Factory::getDbo();

		// Check if columns exists in database
		function qt($n) {
				return(Factory::getDBO()->quote($n));
		}

		$query = 'SHOW COLUMNS FROM #__'.$table.' WHERE Field IN ('.implode(",", array_map("qt", $columns)).')';
		$db->setQuery($query);
		$rows = $db->loadColumn(0);

		// Abort if we don't have any rows
		if (!$rows) {
				return;
		}

		// Let's remove the columns
		$q = "";
		foreach ($rows as $key => $column) {
				$comma = (($key+1) < count($rows)) ? "," : "";
				$q .= "drop ".$this->db->escape($column).$comma;
		}

		$query = "alter table #__".$table." $q";

		$db->setQuery($query);
		$db->execute();
	}
	
	public static function Fetch($table, $columns = "*", $where = null, $singlerow = false)
	{
		if (!$table) {
			return;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery(true);

		$query->select($columns)
					->from("#__$table");

		if (isset($where)) {
			$query->where("$where");
		}

		$db->setQuery($query);

		return ($singlerow) ? $db->loadObject() : $db->loadObjectList();
	}

	
}
