<?php
/*
 * @package         Logical-Arts
 * @version         24.4.0
 *  
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2023 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */


defined('_JEXEC') or die;
if(!class_exists('DatabaseHelper')) {
	require_once __DIR__ . '/DatabaseHelper.php';
}
if(!class_exists('FilesHelper')) {
	require_once __DIR__ . '/FilesHelper.php';
}


use Joomla\CMS\Factory;
use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\CMS\Installer\InstallerScriptInterface;
use Joomla\CMS\Language\Text;


class LartsInstallerBase implements InstallerScriptInterface
{
	const MODULE_TYPE_ADMIN = 1;
	const MODULE_TYPE_SITE = 0;
	
	public $extension_name = '';
	public $extension_type = '';
	public $element = '';
	public $plugin_folder = 'system';
	public $module_type = self::MODULE_TYPE_SITE;
	public $module_position = 'status';
	public $message_name = '';
	
	public $client_id = 1;
	public $install_type = 'install';
	public $show_message = true;
	public $autopublish = true;
	public $db = null;
	public $app = null;
	public $new_xmldata = null;
	public $installed_xmldata = null;
  public $installed_extension_id = 0;
	public $installedVersionInfo = null;
  public $installed_package_id = 0;
  public $vendor_webpage = 'http://www.logical-arts-extensions.com';
  public $installerLanguage = 'lartsinstaller';
	public $deleteInstaller = false;

	//public function __construct(Installer $parent, DatabaseDriver $db, array $options = [])
	public function __construct()
	{
		$this->db = Factory::getDbo();
		$this->app = Factory::getApplication();

		if(empty($this->element)) {
			$this->element = $this->getElementName();
		}
		$this->installedVersionInfo = $this->getInstalledVersionInfo();

		Factory::getLanguage()->load($this->installerLanguage, __DIR__);
	}

	// Overridable method called before installation
	public function onBeforeInstall()
	{
		if (!$this->canInstall()) {
			return false;
		}
		return true;
	}

	// Overridable method called after installation
	public function onAfterInstall()
	{
		return true;
	}

	public function install(InstallerAdapter $adapter): bool
	{
		return true;
	}

	public function update(InstallerAdapter $adapter): bool
	{
		return true;
	}

	public function uninstall(InstallerAdapter $adapter): bool
	{
    // ensure plugins shutdown
		if($this->extension_type == 'plugin') {
      $this->publishPlugin(false);
    }
		
		return true;
	}

	public function preflight(string $type, InstallerAdapter $adapter): bool
	{
		// if we're called with 'uninstall', return
		if (!in_array($type, array('install', 'update'))) {
			return true;
		}

		if ($this->isInstalled()) {
			$this->install_type = 'update';
		}

		if ($this->onBeforeInstall() === false) {
			return false;
		}

		return true;
	}

	public function postflight(string $type, InstallerAdapter $adapter): bool
	{
		// if we're called with 'uninstall', return
		if (!in_array($type, array('install', 'update'))) {
			return true;
		}

    if($this->installed_extension_id == 0) {
      $extinfo = $this->lookupExtensionId();
      if(!empty($extinfo)) {
        $this->installed_extension_id = $extinfo->extension_id;
        $this->installed_package_id = $extinfo->package_id;
      }
    }

		if ($this->onAfterInstall() === false) {
			return false;
		}

    // append versiontype to extension params
	  $vi = $this->getNewVersionInfo();

		if(!empty($vi['type'])) {
			$params['versiontype'] = $vi['type'];
			$this->add2ManifestCache($this->installed_extension_id, $params);
		}
    
    $id = ($this->installed_package_id > 0) ? $this->installed_package_id : $this->installed_extension_id;
    $this->clearUpdateSites($id);


		// publish extension on fresh install
		if ($type == 'install' && $this->autopublish) {
			$this->publishExtension();
		}

		if ($this->show_message) {
			$this->addInstalledMessage();
		}

		Factory::getCache()->clean('com_plugins');
		Factory::getCache()->clean('_system');
		
		if($this->deleteInstaller) {
			$this->deleteInstaller();
		}

		return true;
	}

	protected function deleteInstaller()
	{
		$path = $this->getTargetFolder() . '/install';
		FilesHelper::delTree($path);
	}
	
	public function getExtensionVendorLink()
	{
		$link = $this->vendor_webpage .'/'. $this->extension_name;
		return $link;
	}

	public function isInstalled()
	{
		if (!is_file($this->getInstalledXMLFile())) {
			return false;
		}

		$result = $this->lookupExtensionId();

		return empty($result) ? false : true;
	}

  public function lookupExtensionId()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName(['extension_id', 'package_id']))
			->from('#__extensions')
			->where($this->db->quoteName('type') . ' = ' . $this->db->quote($this->extension_type))
			->where($this->db->quoteName('element') . ' = ' . $this->db->quote($this->element));
		$this->db->setQuery($query, 0, 1);
		return $this->db->loadObject();
	}

	public function clearUpdateSites($extension_id)
	{
    // select all update site entries
		$query = $this->db->getQuery(true)
			->select('*')
			->from('#__update_sites_extensions')
			->where($this->db->quoteName('extension_id') . ' = ' . $this->db->quote($extension_id));
		$this->db->setQuery($query);
		$updatesites = $this->db->loadObjectList();
    
    if(empty($updatesites)) {
      return;
    }

    $aids = [];
    foreach($updatesites as $us) {
      $aids[] = $us->update_site_id;
    }

    // drop all sites entries
    unset($query);
    $query = $this->db->getQuery(true)
      ->delete('#__update_sites')
      ->where($this->db->quoteName('update_site_id') . ' IN (' . implode(',', $aids) . ')');
    $this->db->setQuery($query);
    $this->db->execute();
    
    // drop all update site entries
    unset($query);
    $query = $this->db->getQuery(true)
      ->delete('#__update_sites_extensions')
			->where($this->db->quoteName('extension_id') . ' = ' . $this->db->quote($extension_id));
		$this->db->setQuery($query);
		$this->db->execute();
	}

	public function getElementName()
	{
		switch ($this->extension_type) {
		case 'component':
			return 'com_' . $this->extension_name;
		case 'module':
			return 'mod_' . $this->extension_name;
		case 'plugin':
		default:
			return $this->extension_name;
		}
	}

	public function getTargetFolder()
	{
		switch ($this->extension_type) {
		case 'plugin' :
			$folder = !empty($this->plugin_folder) ? $this->plugin_folder .'/' : '';
			return JPATH_SITE . '/plugins/' .$folder. $this->element;

		case 'component' :
			return JPATH_ADMINISTRATOR . '/components/' . $this->element;

		case 'module' :
			if($this->module_type == self::MODULE_TYPE_SITE) {
				return JPATH_SITE . '/modules/' . $this->element;
			}
			else {
				return JPATH_ADMINISTRATOR . '/modules/' . $this->element;
			}
		case 'library' :
			return JPATH_SITE . '/libraries/' . $this->element;
		}
	}

	public function getInstalledXMLFile()
	{
		return $this->getXMLFilename($this->getTargetFolder());
	}

	public function getNewXMLFile()
	{
		return $this->getXMLFilename(realpath(__DIR__ .'/../'));
	}

	public function getXMLFilename($folder)
	{
		return $folder . '/' . $this->extension_name . '.xml';
	}


	public function publishExtension()
	{
		switch ($this->extension_type) {
    case 'plugin' :
      $this->publishPlugin(true);
      break;
		case 'module' :
			$this->publishModule();
      break;
    }
	}

	public function publishPlugin($bEnabled)
	{
		$query = $this->db->getQuery(true)
			->update('#__extensions')
			->set($this->db->quoteName('enabled') . ' = '. ($bEnabled ? '1' : '0'))
			->where($this->db->quoteName('type') . ' = ' . $this->db->quote('plugin'))
			->where($this->db->quoteName('element') . ' = ' . $this->db->quote($this->element))
			->where($this->db->quoteName('folder') . ' = ' . $this->db->quote($this->plugin_folder));
		$this->db->setQuery($query);
		$this->db->execute();
	}

	public function publishModule()
	{
		// Get module id
		$query = $this->db->getQuery(true)
			->select('id')
			->from('#__modules')
			->where($this->db->quoteName('module') . ' = ' . $this->db->quote('mod_' . $this->extension_name))
			->where($this->db->quoteName('client_id') . ' = ' . (int) $this->client_id);
		$this->db->setQuery($query, 0, 1);
		$id = $this->db->loadResult();

		if (!$id) {
			return;
		}

		// check if module is already in the modules_menu table (meaning is is already saved)
		$query->clear()
			->select('moduleid')
			->from('#__modules_menu')
			->where($this->db->quoteName('moduleid') . ' = ' . (int) $id);
		$this->db->setQuery($query, 0, 1);
		$exists = $this->db->loadResult();

		if ($exists) {
			return;
		}

		// Get highest ordering number in position
		$query->clear()
			->select('ordering')
			->from('#__modules')
			->where($this->db->quoteName('position') . ' = ' . $this->db->quote($this->module_position))
			->where($this->db->quoteName('client_id') . ' = ' . (int) $this->client_id)
			->order('ordering DESC');
		$this->db->setQuery($query, 0, 1);
		$ordering = $this->db->loadResult();
		$ordering++;

		// publish module and set ordering number
		$query->clear()
			->update('#__modules')
			->set($this->db->quoteName('published') . ' = 1')
			->set($this->db->quoteName('ordering') . ' = ' . (int) $ordering)
			->set($this->db->quoteName('position') . ' = ' . $this->db->quote($this->module_position))
			->where($this->db->quoteName('id') . ' = ' . (int) $id);
		$this->db->setQuery($query);
		$this->db->execute();

		// add module to the modules_menu table
		$query->clear()
			->insert('#__modules_menu')
			->columns(array($this->db->quoteName('moduleid'), $this->db->quoteName('menuid')))
			->values((int) $id . ', 0');
		$this->db->setQuery($query);
		$this->db->execute();
	}

	public function addInstalledMessage()
	{
	  $vi = $this->getNewVersionInfo();
		$vt = empty($vi['type']) ? '' : ' - '. strtoupper($vi['type']);
    $txt = Text::sprintf(
      Text::_($this->install_type == 'update' ? 'LARTS_THE_EXTENSION_HAS_BEEN_UPDATED_SUCCESSFULLY' : 'LARTS_THE_EXTENSION_HAS_BEEN_INSTALLED_SUCCESSFULLY'),
      '<strong>' . Text::_($this->message_name) . ' ' .$vi['version']. $vt.'</strong>'
    );
    
		Factory::getApplication()->enqueueMessage($txt);
	}

	public function getPrefix()
	{
		switch ($this->extension_type) {
		case 'plugin';
			return 'plg_' . strtolower($this->plugin_folder);
		case 'component':
			return 'com';
		case 'module':
			return 'mod';
		case 'library':
			return 'lib';
		default:
			return $this->extension_type;
		}
	}

	public function getFullType()
	{
		return Text::_('LARTS_' . strtoupper($this->getPrefix()));
	}

	public function getNewXMLData($file = '')
	{
		if($this->new_xmldata != null) {
			return $this->new_xmldata;
		}
		
		// load it...
		$file = $file ?: $this->getNewXMLFile();
		if (!is_file($file)) {
			return null;
		}

		$this->new_xmldata = $this->loadElementXMLData($file);
		return $this->new_xmldata;
	}
	
	public function getInstalledXMLData($file = '')
	{
		if($this->installed_xmldata != null) {
			return $this->installed_xmldata;
		}
		
		// load it...
		$file = $file ?: $this->getInstalledXMLFile();
		if (!is_file($file)) {
			return null;
		}

		$this->installed_xmldata = $this->loadElementXMLData($file);
		return $this->installed_xmldata;
	}

	public function getInstalledVersionInfo()
	{
		$xml = $this->getInstalledXMLData();
		if (empty($xml)) {
			return null;
		}

    $vi = ['version' => $xml['version'], 'type' => $xml['versiontype']];
		return $vi;
	}

  public function getNewVersionInfo()
  {
    $vi = ['version' => '0.0.0', 'type' => ''];
    
		$xml = $this->getNewXMLData();
		if (!empty($xml)) {
			$vi['version'] = $xml['version'];
			$vi['type'] = $xml['versiontype'];
		}

		return $vi;
  }

	public function getNewVersion()
	{
		$vi = $this->getNewVersionInfo();
		return $vi['version'];
	}

	// Checks wether the extension can be installed or not
	public function canInstall()
	{
		// The extension is not installed yet. Accept Install.
		if ($this->installedVersionInfo == null) {
			return true;
		}

		// The free version is installed. Accept install of pro.
		if (strtolower($this->installedVersionInfo['type']) == 'free') {
			return true;
		}

		// installed version is pro
		// prevent installing free over pro version
		$viNew = $this->getNewVersionInfo();

		// deny update FREE -> PRO
		if (strtolower($viNew['type']) == 'free')
		{
			Factory::getApplication()->enqueueMessage(Text::_('LARTS_ERROR_PRO_TO_FREE'), 'error');
			Factory::getApplication()->enqueueMessage(
				html_entity_decode(
					Text::sprintf(
						'LARTS_ERROR_UNINSTALL_FIRST', 
						$this->getExtensionVendorLink(),
						Text::_($this->message_name)
					)
				), 'error'
			);	

			return false;
		}

		return true;
	}

	// Checks if current version is newer than the installed one
	public function isNewer()
	{
		if($this->installedVersionInfo == null) {
			return true;
		}
		$viNew = $this->getNewVersionInfo();
		return version_compare($this->installedVersionInfo['version'], $viNew['version'], '<=');
	}

	// Re-orders plugin after passed array of plugins
	public function pluginOrderAfter($lowerPluginOrder)
	{
		if (!is_array($lowerPluginOrder) || !count($lowerPluginOrder)) {
			return;
		}

		$db = $this->db;

		// Get plugins max order
		$query = $db->getQuery(true);
		$query
				->select($db->quoteName('b.ordering'))
				->from($db->quoteName('#__extensions', 'b'))
				->where($db->quoteName('b.element') . ' IN ("'.implode("\",\"",$lowerPluginOrder).'")')
				->order('b.ordering desc');

		$db->setQuery($query);
		$maxOrder = $db->loadResult();

		if (is_null($maxOrder)) {
				return;
		}

		// Get plugin details
		$query->clear()
					->select(array($db->quoteName('extension_id'), $db->quoteName('ordering')))
					->from($db->quoteName('#__extensions'))
					->where($db->quoteName('element') . ' = ' . $db->quote($this->element));

		$db->setQuery($query);
		$pluginInfo = $db->loadObject();

		if (!isset($pluginInfo->ordering) || $pluginInfo->ordering > $maxOrder)
		{
				return;
		}

		// Update the new plugin order
		$object = new stdClass();
		$object->extension_id = $pluginInfo->extension_id;
		$object->ordering = ($maxOrder + 1);

		try {
			$db->updateObject('#__extensions', $object, 'extension_id');
		} catch (\Exception $e) {
			return $e->getMessage();
		}
  }

 	public function loadManifestCache($extension_id)
	{
		$db = Factory::getDbo();

		// read the existing component manifest_cache
		$query = $db->getQuery(true)
                ->select('manifest_cache')
                ->from($db->quoteName('#__extensions'))
                ->where($db->quoteName('extension_id') . ' = ' . $db->quote($extension_id));
		$db->setQuery($query);
		$result = $db->loadResult();
		if(empty($result)) {
			return false;
		}
		$cache = json_decode($result, true);
    return $cache;
  }

  public function loadParams($extension_id)
	{
		$db = Factory::getDbo();

		// read the existing component manifest_cache
		$query = $db->getQuery(true)
                ->select('params')
                ->from($db->quoteName('#__extensions'))
                ->where($db->quoteName('extension_id') . ' = ' . $db->quote($extension_id));
		$db->setQuery($query);
		$cache = json_decode($db->loadResult(), true);
    return $cache;
  }
  
	public function add2ManifestCache($extension_id, $param_array)
	{
		if (empty($param_array)) {
			return;
		}

		$params = $this->loadManifestCache($extension_id);
		if($params === false) {
			return false;
		}

		// add the new variable(s) to the existing ones
		$params = array_merge($params, $param_array);

		// store the combined new and existing values back as a JSON string
		$paramsString = json_encode($params);

		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__extensions'))
			->set($db->quoteName('manifest_cache') . ' = ' . $db->quote($paramsString))
			->where($db->quoteName('extension_id') . ' = ' . $db->quote($extension_id));
		$db->setQuery($query);
		$db->execute();
	}     
	
	public static function loadElementXMLData($file)
	{
		if (!file_exists($file)) {
			return false;
		}

		// Read the file to see if it's a valid component XML file
		$xml = simplexml_load_file($file);
		if (!$xml) {
			return false;
		}

		// Check for a valid XML root tag.

		// Extensions use 'extension' as the root tag.  Languages use 'metafile' instead

		$name = $xml->getName();
		if ($name !== 'extension') {
			unset($xml);
			return false;
		}

		$data = [];

		$data['name'] = (string) $xml->name;
    $data['type'] = (string) $xml->attributes()->type;
		$data['version']     = (string) $xml->version;
		$data['description'] = (string) $xml->description;
		
		$data['namespace'] = isset($xml->namespace) ? (string)$xml->namespace : '';
		$data['versiontype'] = isset($xml->versiontype) ? strtolower((string)$xml->versiontype) : '';
	
		return $data;
	}

	public static function deleteSystemLanguageFiles($searchname)
	{
		$paths = [ JPATH_ROOT . '/language/', 
							 JPATH_ADMINISTRATOR . '/language/'
						 ];

		foreach($paths as $path)
		{
			// Create a Recursive Directory Iterator
			$directoryIterator = new RecursiveDirectoryIterator($path);

			// Wrap the Directory Iterator with a Recursive Iterator Iterator
			$iterator = new RecursiveIteratorIterator($directoryIterator);

			$matchingFiles = [];
			// Iterate through all files and directories recursively
			foreach ($iterator as $file) {
				// Check if the current item is a file and contains the search string in the filename
				if ($file->isFile() && strpos($file->getFilename(), $searchname) !== false) {
					$matchingFiles[] = $file->getRealPath();
				}
			}

			foreach ($matchingFiles as $file) {
				unlink($file);
			}
		}
	}
	
}



