<?php
/**
 * @package         Captcha free
 * @version         24.4.0
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2023 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\Captcha\LartsCaptcha\lib\Base;
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\Filesystem\Folder;


class Logger
{
  public static $enabled = null;
  
  public function __construct() {
		self::$enabled = false;
  }
  
	public static function enable($enable = true) {
		self::$enabled = $enable;
  }
  
	public static function _($id, $data, $order = false)
	{
		if(!self::$enabled) {
			return;
		}

		$path = Factory::getConfig()->get('log_path') . '/larts_' . strtolower($id) . '.php';

		$output = [];

		if(!is_file($path))
		{
			Folder::create(dirname($path));
			$output[] = '<?php die(\'Forbidden.\'); ?>';
			$output[] = '# Log: ' . ucwords(str_replace('_', ' ', $id));
			$output[] = '';
		}

		$date = new Date;

		$output[] = '[' . $date->format('Y-m-d H:i:s') . ']';
		$output[] = self::generateOutputFromData($data, $order);

		file_put_contents($path, implode("\n", $output) . "\n", FILE_APPEND);
	}

	private static function generateOutputFromData($data, $order = false, $indent_count = 1)
	{
		$output = [];

		$data = (array) $data;

		if ($order)
		{
			ksort($data);
		}

		foreach ($data as $key => $value)
		{
			$output[] = self::generateOutputFromKeyValue($key, $value, $order, $indent_count);
		}

		return implode("\n", $output);
	}

	private static function generateOutputFromKeyValue($key, $value, $order = false, $indent_count = 1)
	{
		$indent = str_repeat(' ', $indent_count * 4);

		if (is_array($value) || is_object($value))
		{

			return $indent . '[' . $key . '] ' . "\n"
				. self::generateOutputFromData($value, $order, $indent_count + 1);
		}

		return $indent . '[' . $key . '] ' . $value;
	}
}
