<?php
/**
 * @package         Captcha free
 * @version         24.4.0
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2023 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 * 
 * Based on https://www.codeproject.com/Articles/26595/CAPTCHA-Image-in-PHP by Mohammad Dayyan
 */

namespace Larts\Plugin\Captcha\LartsCaptcha\lib;
defined('_JEXEC') or die;

use Larts\Plugin\Captcha\LartsCaptcha\lib\Rect;

class ImageGenerator
{
	private $showLine = true;
	private $applyWave = true;
	private $font_file = __DIR__ .'/tahomabd.ttf';
	private $font_size = 33;
	private $output_path = '';

	private $Colors =  array (	'0' => '145',
								'1' => '204',
								'2' => '177',
								'3' => '184',
								'4' => '199',
								'5' => '255');

	public function __construct()
	{
	}

	public function setPath($path) {
		$this->output_path = $path;
	}
	
	public function generate($text, int $width = 150, int $height = 50)
	{
		$width = max(100, $width);
		$height = max(40, $height);
				
		//background image
		$image = imagecreatetruecolor($width, $height);
		if($image === false) {
			return false;
		}
		$bg = imagecolorallocate($image, 255, 255, 255);
		imagefill($image, 10, 10, $bg);

		for ($x=0; $x < $width; $x++)
		{
			for ($y=0; $y < $height; $y++)
			{
				$random = mt_rand(0 , 5);
				$temp_color = imagecolorallocate($image, $this->Colors["$random"], $this->Colors["$random"], $this->Colors["$random"]);
				imagesetpixel( $image, $x, $y , $temp_color );
			}
		}

		$char_color = imagecolorallocatealpha($image, 0, 0, 0, 90);

		$len = strlen($text);
		$pos = [];
		$xOld = 5;
		$yOld = (int)($height / 2);
		$xmin = 10;
		$xmax = $width - 10;
		$ymin = $this->font_size + 10;
		$ymax = $height;
		
		$rectChar = new Rect;
		for($i = 0; $i < $len; $i++) {
			$xrand = $xOld + mt_rand(-5, 10);
			$xrand = max($xmin, min($xmax, $xrand));

			$yrand = $yOld + mt_rand(-10, 10);
			$yrand = max($ymin, min($ymax, $yrand));
			$angle = mt_rand(-20, 20);

			$box = imageftbbox($this->font_size, $angle, $this->font_file, $text[$i]);
			$rectChar->set((int)min($box[0], $box[6]), (int)min($box[7], $box[5]), 
										(int)max($box[2], $box[4]), (int)max($box[1], $box[3]));

			$rectChar->moveToXY($xrand, $yrand - $rectChar->height);

			$rectChar->moveToXY(0, 0);
			$xOld = $xrand + $rectChar->width;
			$yOld = $yrand;

			$pos[] = [ $xrand, $yrand, $angle, $rectChar->width, $rectChar->height ];
		}

		$bb = new Rect(-999, -999, 999, 999);
		for($i = 0; $i < $len; $i++) {
			$bb->setLeft(min($bb->x, $pos[$i][0]));
			$bb->setTop(min($bb->y, $pos[$i][1] - $pos[$i][4]));
			$bb->setRight(max($bb->getRight(), $pos[$i][0] + $pos[$i][3]));
			$bb->setBottom(max($bb->getBottom(), $pos[$i][1]));
		}

		$offsetx = (int)(($width - $bb->width) / 2);
		$offsety = (int)(($height - $bb->height) / 2);
		$offsetx -= $bb->x;
		$offsety -= $bb->y;

		for($i = 0; $i < $len; $i++) {
			imagefttext($image, $this->font_size, $pos[$i][2], 
						$pos[$i][0] + $offsetx, $pos[$i][1] + $offsety, 
						$char_color, $this->font_file, $text[$i]);
		}

		// apply wave
		if ($this->applyWave) {
			$image = $this->apply_wave($image, $width, $height);
		}
			
		// lines
		if ($this->showLine)
		{
			$x = mt_rand(8, 15);
			$randdist = mt_rand(8, 20);
			for ($x = 0; $x < $width; ) {
				imageline($image, $x, 0, $x + $randdist, $height, $char_color);
				imageline($image, $x, 0, $x - $randdist, $height, $char_color);
				$x += mt_rand(8 , 12);
				$randdist = mt_rand(8, 20);
			}
		}
			
		// write image
		$filepath = $this->output_path. '/img_'. $text. '.png';
		if(!imagepng($image, $filepath)) {
			$filepath = false;
		}
		imagedestroy($image);
		
		return $filepath;
	}

	private function apply_wave($image, $width, $height)
	{		
		$x_period = 10;
		$y_period = 10;
		$y_amplitude = 5;
		$x_amplitude = 5;
		
		$xp = $x_period*rand(1,3);
		$k = mt_rand(0, 100);
		for ($a = 0; $a < $width; $a++) {
			imagecopy($image, $image, $a-1, (int)(sin($k+$a/$xp)*$x_amplitude), $a, 0, 1, $height);
		}
		$yp = $y_period*rand(1,2);
		$k = mt_rand(0, 100);
		for ($a = 0; $a < $height; $a++) {
			imagecopy($image, $image, (int)(sin($k+$a/$yp)*$y_amplitude), $a-1, 0, $a, $width, 1);
		}
		
		return $image;
	}
}

