<?php

/**
 * @package         Captcha free
 * @version         24.4.0
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2023 Logical-Arts All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\Captcha\LartsCaptcha\lib;

class Rect
{
	public $x, $y;
	public $width, $height;

	public function __construct($width = 0, $height = 0, $x = 0, $y = 0)
	{
		$this->width = $width;
		$this->height = $height;
		$this->x = $x;
		$this->y = $y;
	}

	// Getter methods
	public function getWidth() {
		return $this->width;
	}

	public function getHeight()
	{
		return $this->height;
	}

	public function getX()
	{
		return $this->x;
	}

	public function getY()
	{
		return $this->y;
	}

	// Setter methods
	public function setWidth($width) {
		$this->width = $width;
	}
	public function setHeight($height) {
		$this->height = $height;
	}

	public function setX($x) {
		$this->x = $x;
	}
	public function setY($y) {
		$this->y = $y;
	}

	public function setTop($top) {
		$this->y = $top;
	}
	public function setLeft($left) {
		$this->x = $left;
	}
	public function setBottom($bottom) {
		$this->height  = $bottom - $this->y;
	}
	public function setRight($right) {
		$this->width = $right - $this->x;
	}		
	public function getTop() {
		return $this->y;
	}
	public function getLeft() {
		return $this->x;
	}
	public function getBottom() {
		return $this->y + $this->height;
	}
	public function getRight() {
		return $this->x + $this->width;
	}		
	public function moveToXY($x, $y) {
		$this->x = $x;
		$this->y = $y;
	}

	// Offset methods
	public function offsetX($offset) {
		$this->x += $offset;
	}
	public function offsetY($offset) {
		$this->y += $offset;
	}
	public function offsetXY($offx, $offy) {
		$this->x += $offx;
		$this->y += $offy;
	}

	// Set position and size based on two points
	public function set($x1, $y1, $x2, $y2) {
			$this->x = min($x1, $x2);
			$this->y = min($y1, $y2);
			$this->width = abs($x2 - $x1);
			$this->height = abs($y2 - $y1);
	}
	
	// Normalize function
	public function normalize()
	{
		if ($this->width < 0) {
			$this->x -= $this->width;
			$this->width = abs($this->width);
		}
		if ($this->height < 0) {
			$this->y -= $this->height;
			$this->height = abs($this->height);
		}
	}
}
